<?php

namespace App\Http\Requests\API\Container;

use App\Http\Requests\RuleHelper;
use App\Models\Enums\FileObjectType;
use App\Models\Enums\FolderObjectType;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Validation\Rule;

class ApprovalMoveFileRequest extends FormRequest
{
    use RuleHelper;

    /**
     * Determine if the user is authorized to make this request.
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, \Illuminate\Contracts\Validation\ValidationRule|array<mixed>|string>
     */
    public function rules(): array
    {
        return [
            "target_folder_id" => [
                'required',
                Rule::exists('folders', 'id')
                    ->where($this->userCompanyWhere())
                    ->where("object_type", FolderObjectType::CONTAINER)
                    ->where("object_id", $this->route("container")),
            ],
            "top_folder_ids.*" => [
                'required',
                Rule::exists('folders', 'id')
                    ->where($this->userCompanyWhere())
                    ->whereNot("id", $this->get("target_folder_id"))
                    ->where("parent_id", 0)
                    ->where("object_type", FolderObjectType::CONTAINER)
                    ->where("object_id", $this->route("container")),
            ],
            "top_file_ids.*" => [
                'required',
                Rule::exists('files', 'id')
                    ->where($this->userCompanyWhere())
                    ->where("folder_id", 0)
                    ->where("object_type", FileObjectType::CONTAINER)
                    ->where("object_id", $this->route("container")),
            ]
        ];
    }
}
