<?php

namespace App\Models;

use App\Models\Enums\ApprovalMode;
use App\Models\Scopes\CompanyScope;
use EloquentFilter\Filterable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Auth;

class Approval extends Model
{
    use HasFactory, SoftDeletes, Filterable;

    protected $fillable = [
        'approval_flow_id',
        'object_type',
        'object_id',
        'node_level',
        'approval_mode',
        'users',
        'remark',
        'created_by',
        'status',
        'company_id'
    ];

    protected static function booted(): void
    {
        static::addGlobalScope(new CompanyScope);
    }

    public function approvalFlow()
    {
        return $this->belongsTo(ApprovalFlow::class)->withTrashed();
    }

    public function createdBy()
    {
        return $this->belongsTo(User::class, 'created_by');
    }

    public function scopeAllowed(Builder $query, bool $isAction = false): void
    {
        $query->where(function(Builder $query) {
            $query->where("users", 'like', '%,'.Auth::id().',%')
                ->whereIn("approval_mode", [
                    ApprovalMode::COUNTERSIGNATURE->value,
                    ApprovalMode::ALTERNATIVE_APPROVAL->value,
                ]);
        })->orWhere(function (Builder $query) {
            $query->where("users", 'like', ','.Auth::id().',%')
                ->whereIn("approval_mode", [
                    ApprovalMode::SEQUENTIAL_APPROVAL->value,
                ]);
        })->when(! $isAction, fn(Builder $query) => $query->orWhere("created_by", Auth::id()));
    }
}
